import { StateEffectType, StateField, Extension } from '@codemirror/state';
import { EditorView } from '@codemirror/view';

/**
 * Type representing a Parinfer mode.
 */
type ParinferMode = "smart" | "indent" | "paren";
type ParinferExtensionConfig = {
    enabled?: boolean;
    mode?: ParinferMode;
};
declare const setConfigEffect: StateEffectType<ParinferExtensionConfig>;
declare const configField: StateField<ParinferExtensionConfig | null>;
/**
 * Updates the editor's extension configuration.
 * @param view the editor view
 * @param config new configuration for the Parinfer extension
 */
declare function configureParinfer(view: EditorView, config: ParinferExtensionConfig): void;
/**
 * Switches the Parinfer mode for the provided editor view.
 * @param view the editor view
 * @param mode the Parinfer mode to switch to
 */
declare function switchMode(view: EditorView, mode: ParinferMode): void;
/**
 * Disables Parinfer for the provided editor view.
 * @param view the editor view
 */
declare function disableParinfer(view: EditorView): void;
/**
 * Enables Parinfer for the provided editor view.
 * @param view the editor view
 */
declare function enableParinfer(view: EditorView): void;
/**
 * Initialises the Parinfer extension for CodeMirror6.
 * @param initialConfig (optional) the initial configuration for the Parinfer extension
 * @returns the CodeMirror6 Parinfer extension in the form of an array of extensions
 */
declare function parinferExtension(initialConfig?: ParinferExtensionConfig): Extension;

export { type ParinferExtensionConfig, type ParinferMode, configField, configureParinfer, disableParinfer, enableParinfer, parinferExtension, setConfigEffect, switchMode };
